@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # Impacket - Collection of Python classes for working with network protocols.
 #
-# SECUREAUTH LABS. Copyright (C) 2021 SecureAuth Corporation. All rights reserved.
+# SECUREAUTH LABS. Copyright (C) 2022 SecureAuth Corporation. All rights reserved.
 #
 # This software is provided under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
@@ -490,7 +490,7 @@ class DPAPI:
                     entropy = fp2.read()
                     fp2.close()
                 elif self.options.entropy is not None:
-                    entropy = b(self.options.entropy) + b'\x00'
+                    entropy = b(self.options.entropy)
                 else:
                     entropy = None
 
