@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # Impacket - Collection of Python classes for working with network protocols.
 #
-# SECUREAUTH LABS. Copyright (C) 2021 SecureAuth Corporation. All rights reserved.
+# SECUREAUTH LABS. Copyright (C) 2022 SecureAuth Corporation. All rights reserved.
 #
 # This software is provided under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
@@ -9,25 +9,11 @@
 #
 from __future__ import division
 from __future__ import print_function
-import sys
-from six import PY2
-sys.path.insert(0,"../..")
-
-#Impact test version
-try:
-    from impacket import IP6_Address, IP6, ImpactDecoder, IP6_Extension_Headers
-except:
-    pass
-
-#Standalone test version
-try:
-    import sys
-    sys.path.insert(0,"../..")
-    import IP6_Address, IP6, ImpactDecoder, IP6_Extension_Headers
-except:
-    pass
-
 import unittest
+from six import PY2
+
+from impacket import IP6, ImpactDecoder, IP6_Extension_Headers
+
 
 class TestIP6(unittest.TestCase):
     def string_to_list(self, bytes):
@@ -624,5 +610,6 @@ class TestIP6(unittest.TestCase):
         self.assertEqual(padn_option_type, 1, "Simple Hop By Hop Parsing - Incorrect option type")
         self.assertEqual(padn_option_length, 12, "Simple Hop By Hop Parsing - Incorrect option size")
 
-suite = unittest.TestLoader().loadTestsFromTestCase(TestIP6)
-unittest.main(defaultTest='suite')
+
+if __name__ == '__main__':
+    unittest.main(verbosity=1)
