@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # Impacket - Collection of Python classes for working with network protocols.
 #
-# SECUREAUTH LABS. Copyright (C) 2021 SecureAuth Corporation. All rights reserved.
+# SECUREAUTH LABS. Copyright (C) 2022 SecureAuth Corporation. All rights reserved.
 #
 # This software is provided under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
@@ -688,10 +688,4 @@ class SimpleSMBServer2FuncTests(SimpleSMBServerFuncTests):
 
 
 if __name__ == "__main__":
-    loader = unittest.TestLoader()
-    suite = unittest.TestSuite()
-    suite.addTests(loader.loadTestsFromTestCase(SMBServerUnitTests))
-    suite.addTests(loader.loadTestsFromTestCase(SimpleSMBServerFuncTests))
-    suite.addTests(loader.loadTestsFromTestCase(SimpleSMBServer2FuncTests))
-    suite.addTests(loader.loadTestsFromTestCase(SimpleSMBServer2FuncTestsClientFallBack))
-    unittest.main(defaultTest='suite')
+    unittest.main(verbosity=1)
