@@ -1,6 +1,6 @@
 # Impacket - Collection of Python classes for working with network protocols.
 #
-# SECUREAUTH LABS. Copyright (C) 2021 SecureAuth Corporation. All rights reserved.
+# SECUREAUTH LABS. Copyright (C) 2022 SecureAuth Corporation. All rights reserved.
 #
 # This software is provided under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
@@ -44,29 +44,33 @@
 from __future__ import division
 from __future__ import print_function
 
-import base64
-import unittest
 import zlib
-
-try:
-    import ConfigParser
-except ImportError:
-    import configparser as ConfigParser
+import base64
+import pytest
+import unittest
+from tests import RemoteTestCase
 
 from impacket.dcerpc.v5.dcom import wmi
 from impacket.dcerpc.v5.dtypes import NULL
 from impacket.dcerpc.v5.dcomrt import DCOMConnection
 
 
-class WMITests(unittest.TestCase):
-    def tes_activation(self):
+@pytest.mark.remote
+class WMITests(RemoteTestCase, unittest.TestCase):
+
+    def setUp(self):
+        super(WMITests, self).setUp()
+        self.set_transport_config()
+
+    @pytest.mark.xfail
+    def test_activation(self):
         dcom = DCOMConnection(self.machine, self.username, self.password, self.domain, self.lmhash, self.nthash)
-        dcom.CoCreateInstanceEx(wmi.CLSID_WbemLevel1Login,wmi.IID_IWbemLoginClientID)
+        dcom.CoCreateInstanceEx(wmi.CLSID_WbemLevel1Login, wmi.IID_IWbemLoginClientID)
         dcom.disconnect()
 
     def test_IWbemLevel1Login_EstablishPosition(self):
         dcom = DCOMConnection(self.machine, self.username, self.password, self.domain, self.lmhash, self.nthash)
-        iInterface = dcom.CoCreateInstanceEx(wmi.CLSID_WbemLevel1Login,wmi.IID_IWbemLevel1Login)
+        iInterface = dcom.CoCreateInstanceEx(wmi.CLSID_WbemLevel1Login, wmi.IID_IWbemLevel1Login)
         iWbemLevel1Login = wmi.IWbemLevel1Login(iInterface)
         resp = iWbemLevel1Login.EstablishPosition()
         print(resp)
@@ -74,7 +78,7 @@ class WMITests(unittest.TestCase):
 
     def test_IWbemLevel1Login_RequestChallenge(self):
         dcom = DCOMConnection(self.machine, self.username, self.password, self.domain, self.lmhash, self.nthash)
-        iInterface = dcom.CoCreateInstanceEx(wmi.CLSID_WbemLevel1Login,wmi.IID_IWbemLevel1Login)
+        iInterface = dcom.CoCreateInstanceEx(wmi.CLSID_WbemLevel1Login, wmi.IID_IWbemLevel1Login)
         iWbemLevel1Login = wmi.IWbemLevel1Login(iInterface)
         try:
             resp = iWbemLevel1Login.RequestChallenge()
@@ -87,7 +91,7 @@ class WMITests(unittest.TestCase):
 
     def test_IWbemLevel1Login_WBEMLogin(self):
         dcom = DCOMConnection(self.machine, self.username, self.password, self.domain, self.lmhash, self.nthash)
-        iInterface = dcom.CoCreateInstanceEx(wmi.CLSID_WbemLevel1Login,wmi.IID_IWbemLevel1Login)
+        iInterface = dcom.CoCreateInstanceEx(wmi.CLSID_WbemLevel1Login, wmi.IID_IWbemLevel1Login)
         iWbemLevel1Login = wmi.IWbemLevel1Login(iInterface)
         try:
             resp = iWbemLevel1Login.WBEMLogin()
@@ -100,44 +104,44 @@ class WMITests(unittest.TestCase):
 
     def test_IWbemLevel1Login_NTLMLogin(self):
         dcom = DCOMConnection(self.machine, self.username, self.password, self.domain, self.lmhash, self.nthash)
-        iInterface = dcom.CoCreateInstanceEx(wmi.CLSID_WbemLevel1Login,wmi.IID_IWbemLevel1Login)
+        iInterface = dcom.CoCreateInstanceEx(wmi.CLSID_WbemLevel1Login, wmi.IID_IWbemLevel1Login)
         iWbemLevel1Login = wmi.IWbemLevel1Login(iInterface)
         resp = iWbemLevel1Login.NTLMLogin('\\\\%s\\root\\cimv2' % self.machine, NULL, NULL)
         print(resp)
         dcom.disconnect()
 
-    def tes_IWbemServices_OpenNamespace(self):
-        # Not working
+    @pytest.mark.xfail
+    def test_IWbemServices_OpenNamespace(self):
         dcom = DCOMConnection(self.machine, self.username, self.password, self.domain, self.lmhash, self.nthash)
-        iInterface = dcom.CoCreateInstanceEx(wmi.CLSID_WbemLevel1Login,wmi.IID_IWbemLevel1Login)
+        iInterface = dcom.CoCreateInstanceEx(wmi.CLSID_WbemLevel1Login, wmi.IID_IWbemLevel1Login)
         iWbemLevel1Login = wmi.IWbemLevel1Login(iInterface)
-        iWbemServices= iWbemLevel1Login.NTLMLogin('//./ROOT', NULL, NULL)
+        iWbemServices = iWbemLevel1Login.NTLMLogin('//./ROOT', NULL, NULL)
         try:
             resp = iWbemServices.OpenNamespace('__Namespace')
             print(resp)
-        except Exception as e:
+        except Exception:
             dcom.disconnect()
             raise
         dcom.disconnect()
 
     def test_IWbemServices_GetObject(self):
         dcom = DCOMConnection(self.machine, self.username, self.password, self.domain, self.lmhash, self.nthash)
-        iInterface = dcom.CoCreateInstanceEx(wmi.CLSID_WbemLevel1Login,wmi.IID_IWbemLevel1Login)
+        iInterface = dcom.CoCreateInstanceEx(wmi.CLSID_WbemLevel1Login, wmi.IID_IWbemLevel1Login)
         iWbemLevel1Login = wmi.IWbemLevel1Login(iInterface)
         iWbemServices= iWbemLevel1Login.NTLMLogin('\\\\%s\\root\\cimv2' % self.machine, NULL, NULL)
         iWbemLevel1Login.RemRelease()
 
-        classObject,_ = iWbemServices.GetObject('Win32_Process')
+        classObject, _ = iWbemServices.GetObject('Win32_Process')
        
         dcom.disconnect()
 
     def test_IWbemServices_ExecQuery(self):
         dcom = DCOMConnection(self.machine, self.username, self.password, self.domain, self.lmhash, self.nthash)
-        iInterface = dcom.CoCreateInstanceEx(wmi.CLSID_WbemLevel1Login,wmi.IID_IWbemLevel1Login)
+        iInterface = dcom.CoCreateInstanceEx(wmi.CLSID_WbemLevel1Login, wmi.IID_IWbemLevel1Login)
         iWbemLevel1Login = wmi.IWbemLevel1Login(iInterface)
-        iWbemServices= iWbemLevel1Login.NTLMLogin('\\\\%s\\root\\cimv2' % self.machine, NULL, NULL)
+        iWbemServices = iWbemLevel1Login.NTLMLogin('\\\\%s\\root\\cimv2' % self.machine, NULL, NULL)
         #classes = [ 'Win32_Account', 'Win32_UserAccount', 'Win32_Group', 'Win32_SystemAccount', 'Win32_Service']
-        classes = [ 'Win32_Service']
+        classes = ['Win32_Service']
         for classn in classes:
             print("Reading %s " % classn)
             try:
@@ -158,21 +162,21 @@ class WMITests(unittest.TestCase):
         dcom.disconnect()
 
     def test_IWbemServices_ExecMethod(self):
-        dcom = DCOMConnection(self.machine, self.username, self.password, self.domain, self.lmhash, self.nthash)        
-        iInterface = dcom.CoCreateInstanceEx(wmi.CLSID_WbemLevel1Login,wmi.IID_IWbemLevel1Login)
+        dcom = DCOMConnection(self.machine, self.username, self.password, self.domain, self.lmhash, self.nthash)
+        iInterface = dcom.CoCreateInstanceEx(wmi.CLSID_WbemLevel1Login, wmi.IID_IWbemLevel1Login)
         iWbemLevel1Login = wmi.IWbemLevel1Login(iInterface)
-        iWbemServices= iWbemLevel1Login.NTLMLogin('\\\\%s\\root\\cimv2' % self.machine, NULL, NULL)
+        iWbemServices = iWbemLevel1Login.NTLMLogin('\\\\%s\\root\\cimv2' % self.machine, NULL, NULL)
 
         #classObject,_ = iWbemServices.GetObject('WinMgmts:Win32_LogicalDisk='C:'')
-        classObject,_ = iWbemServices.GetObject('Win32_Process')
+        classObject, _ = iWbemServices.GetObject('Win32_Process')
         obj = classObject.Create('notepad.exe', 'c:\\', None)
         handle = obj.getProperties()['ProcessId']['value']
         
         iEnumWbemClassObject = iWbemServices.ExecQuery('SELECT * from Win32_Process where handle = %s' % handle)
-        oooo = iEnumWbemClassObject.Next(0xffffffff,1)[0]
+        oooo = iEnumWbemClassObject.Next(0xffffffff, 1)[0]
         #import time
         #time.sleep(5)
-        owner = oooo.Terminate(1)
+        oooo.Terminate(1)
 
         #iEnumWbemClassObject = iWbemServices.ExecQuery('SELECT * from Win32_Group where name = "testGroup0"')
         #oooo = iEnumWbemClassObject.Next(0xffffffff,1)[0]
@@ -200,46 +204,9 @@ class WMITests(unittest.TestCase):
 
         dcom.disconnect()
 
-class TCPTransport(WMITests):
-    def setUp(self):
-        WMITests.setUp(self)
-        configFile = ConfigParser.ConfigParser()
-        configFile.read('dcetests.cfg')
-        self.username = configFile.get('TCPTransport', 'username')
-        self.domain   = configFile.get('TCPTransport', 'domain')
-        self.serverName = configFile.get('TCPTransport', 'servername')
-        self.password = configFile.get('TCPTransport', 'password')
-        self.machine  = configFile.get('TCPTransport', 'machine')
-        self.hashes   = configFile.get('TCPTransport', 'hashes')
-        self.stringBinding = r'ncacn_ip_tcp:%s' % self.machine
-        self.ts = ('8a885d04-1ceb-11c9-9fe8-08002b104860', '2.0')
-        if len(self.hashes) > 0:
-            self.lmhash, self.nthash = self.hashes.split(':')
-        else:
-            self.lmhash = ''
-            self.nthash = ''
 
-class TCPTransport64(WMITests):
-    def setUp(self):
-        WMITests.setUp(self)
-        configFile = ConfigParser.ConfigParser()
-        configFile.read('dcetests.cfg')
-        self.username = configFile.get('TCPTransport', 'username')
-        self.domain   = configFile.get('TCPTransport', 'domain')
-        self.serverName = configFile.get('TCPTransport', 'servername')
-        self.password = configFile.get('TCPTransport', 'password')
-        self.machine  = configFile.get('TCPTransport', 'machine')
-        self.hashes   = configFile.get('TCPTransport', 'hashes')
-        self.stringBinding = r'ncacn_ip_tcp:%s' % self.machine
-        self.ts = ('71710533-BEBA-4937-8319-B5DBEF9CCC36', '1.0')
-        if len(self.hashes) > 0:
-            self.lmhash, self.nthash = self.hashes.split(':')
-        else:
-            self.lmhash = ''
-            self.nthash = ''
+class WMIOfflineTests(unittest.TestCase):
 
-
-class OfflineTests(unittest.TestCase):
     @staticmethod
     def createIWbemClassObject(b64_compressed_obj_ref):
         obj_ref = zlib.decompress(base64.b64decode(b64_compressed_obj_ref))
@@ -378,13 +345,5 @@ class OfflineTests(unittest.TestCase):
 
 
 # Process command-line arguments.
-if __name__ == '__main__':
-    import sys
-    if len(sys.argv) > 1:
-        testcase = sys.argv[1]
-        suite = unittest.TestLoader().loadTestsFromTestCase(globals()[testcase])
-    else:
-        suite = unittest.TestLoader().loadTestsFromTestCase(TCPTransport)
-        suite.addTests(unittest.TestLoader().loadTestsFromTestCase(TCPTransport64))
-        suite.addTests(unittest.TestLoader().loadTestsFromTestCase(OfflineTests))
-    unittest.main(defaultTest='suite')
+if __name__ == "__main__":
+    unittest.main(verbosity=1)
